/*
 * calcbackup - a command line utility to back up variables
 * on a TI89 Titanium calculator over the USB link
 *
 * Copyright (C) 2010 Michael Jordan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "ti_usb.h"
#include "ti_file.h"

int main(int argc, char *argv[]) {
	int return_code = EX_OK;
	bool verbose = false;
	if(argc > 1) {
		string argument;
		argument.append(argv[1]);
		bool help = false;
		bool valid = false;
		if((argument == "-h") || (argument == "--help")) {
			valid = true;
			help = true;
		}
		if((argument == "-v") || (argument == "--verbose")) {
			valid = true;
			verbose = true;
		}
		if((argument == "-q") || (argument == "--quiet")) {
			valid = true;
		}
		if(!valid) help = true;
		if(help) {
			cout <<
					"\ncalcbackup is a utility to back up the files located on a\n" <<
					"TI89 Titanium calculator over the USB link.\n\n" << flush;
			cout <<
					"It will request a listing of the variables on the calculator\n" <<
					"and then proceed to save each variable to a file.  The naming\n" <<
					"convention for the files closely follows TI's naming convention,\n" <<
					"and the files are compatible with TI Connect, so they can be\n" <<
					"reloaded using TI's provided utilities.\n" <<
					"The recommended way to run this software is from an empty\n" <<
					"directory.  The utility will create one file for each variable\n" <<
					"on the calculator.\n" <<
					"This utility runs in two basic modes - user interface and script.\n" <<
					"When script mode is used (-q or --quiet) the utility will\n" <<
					"go about its business quietly and produce NO output to stdout.\n" <<
					"This is provided for use in scripts which will check the return\n"
					"value.\n" <<
					"The user interface mode (-v, or --verbose) outputs information\n" <<
					"at each step of the process so that a user can see the progress\n" <<
					"and note any warnings as they occur.\n\n" <<
					"Command line arguments:\n" <<
					"\t-h, --help - These options will print this help screen\n" <<
					"\t\tthen exit.\n" <<
					"\t-v, --verbose - These options will put the program into\n" <<
					"\t\tuser interface mode.\n" <<
					"\t-q, --quiet - These options will put the program into\n" <<
					"\t\tscript mode.\n" <<
					"\nIn either case this utility will return one of three values:\n" <<
					"\tEX_OK\t\t0\tThe process completed\n" <<
					"\t\t\t\tsuccessfully and all vars were\n" <<
					"\t\t\t\tsaved.\n" <<
					"\tENOTCONN\t107\tNo calculator was located on the\n" <<
					"\t\t\t\tUSB link.\n" <<
					"\tEIO\t\t5\tAt least one of the vars was not\n" <<
					"\t\t\t\tsaved successfully to a file.\n" <<
					flush;
			cout <<
					"Keep in mind that this is, in fact, free software, developed\n" <<
					"independently and therefore comes with no warranty.\n" << flush;
			return return_code;
		}
	} else {
		return_code = EX_USAGE;
		cout << "This utility requires one of the following arguments:\n" <<
				"\t-h, --help\n" <<
				"Or one of the following sets:\n" <<
				"\t-v\n" <<
				"\t-q\n" <<
				"Alternatively -v may be --verbose or -q may be --quiet\n" << flush;
		return return_code;
	}

	if(verbose) cout << "Searching for calculator..." << flush;
	ti_usb calc;

	// Calling is_good() here verifies that there was a calculator located on
	// the USB bus and that we can in fact communicate with it.
	if(calc.is_good()) {
		vector<ti_usb::ls_item_info> items;
		ti_usb::ls_item_info ls_item;

		// We obtain a listing of the vars on the calculator in order to iterate
		// through and request them to be transmitted to us in order to then
		// save them.
		if(verbose) cout << "Calculator located...Getting listing...\n";
		calc.ls(items);

		// Print the list out for the user to see.
		if(verbose) cout << right << setw(13) << "Folder:" <<
			left << setw(8) << "Name" <<
			right << setw(10) << "Size:" << "  Archived Locked Type" <<
			endl << endl;
		for(int i = 0; i < ((int)items.size() - 1); i++) {
			if((i > 0) && verbose) cout << right << setw(4) << i;
			else if(verbose) cout << right << setw(4) << " ";
			if(verbose) cout << right << setw(8) << items[i].parent_folder << ":" <<
				left << setw(8) << items[i].name <<
				right << setw(10) << items[i].size << "  " <<
				boolalpha << left << setw(8) << items[i].archived << " " <<
				items[i].locked << "\t " <<
				calc.typeid_to_string(items[i].type) << endl;
		}

		// Now we print out each individual item as we process so the user can
		// see the status (whether we retrieved the var successfully and then
		// whether we were able to save the var successfully).
		for(int num = 1; num < ((int)items.size() - 1); num++) {
			if(verbose) cout << right << setw(13) << "Folder:" <<
					left << setw(8) << "Name" <<
					right << setw(10) << "Size:" << " Archived Locked Type" << endl;
			if(verbose) cout << right << setw(4) << num <<
					right << setw(8) << items[num].parent_folder << ":" <<
					left << setw(8) << items[num].name <<
					right << setw(10) << items[num].size << " " <<
					boolalpha << left << setw(8) << items[num].archived << " " <<
					items[num].locked << "\t" <<
					calc.typeid_to_string(items[num].type) << endl << endl;
			ls_item = items[num];
			if((ls_item.type & 0xFF) != 0x1F) {
				unsigned char buf[ls_item.size];
				if(verbose) cout << "Getting data..." << flush;
				if(calc.get_var_data(ls_item, buf) == 0) {
					if(verbose) cout << "success - saving...";
					int ret;
					ret = calc.var_save(ls_item, buf, "./");
					if(ret != 0) {
						return_code = EIO;
						if(verbose) cout << "failed - result " << ret << "\n" << flush;
					}
					else if(verbose) cout << "success\n" << flush;
				}
				else {
					return_code = EIO;
					if(verbose) cout << "failed - skipping\n";
				}
			} else if(verbose) cout << "Folder - skipping\n";
		}
		return return_code;
	}
	if(verbose) cout << "No calculator found on the USB bus\n";
	return_code = ENOTCONN;
	return return_code;
}
