/*
 * calcgetscreen - a command line utility to save a screenshot from
 * a TI89 Titanium calculator over the USB link
 *
 * Copyright (C) 2010 Michael Jordan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */
//============================================================================

#include "ti_usb.h"
#include "ti_file.h"

int main(int argc, char *argv[]) {
	int return_code = EX_OK;
	bool verbose = false;
	string filename = "";
	if(argc > 1) {
		string argument;
		argument.append(argv[1]);
		bool help = false;
		bool valid = false;
		if((argument == "-h") || (argument == "--help")) {
			valid = true;
			help = true;
		}
		if((argument == "-q") || (argument == "--quiet")) {
			verbose = false;
			if(argc > 2) filename.append(argv[2]);
			valid = true;
			help = false;
		}
		if((argument == "-v") || (argument == "--verbose")) {
			verbose = true;
			if(argc > 2) filename.append(argv[2]);
			valid = true;
			help = false;
		}
		if(filename == "") filename = "screen01.pbm";
		if(!valid) help = true;
		if(help) {
			cout <<
					"\ncalcgetscreen is a utility to get a screen shot from a\n" <<
					"TI89 Titanium calculator over the USB link.\n\n" << flush;
			cout <<
					"This utility runs in two modes - verbose and quiet.  Quiet\n" <<
					"mode is intended for use in scripts.  The -q or --quiet option\n" <<
					"accomplishes this.  For example:\n" <<
					"\tcalcgetscreen -q [filename]\n\n" <<
					"The filename option is optional - if not added the default\n" <<
					"filename of \'screen01.pbm\' is used and the file is saved\n" <<
					"to the current directory.\n" <<
					"Quiet mode produces no output to stdout and returns one of the\n" <<
					"following values:\n" <<
					"\tEX_OK\t\t0 - The operation completed successfully\n" <<
					"\tEX_USAGE\t64 - The arguments are incorrect\n" <<
					"\tENOTCONN\t107 - No calculator was found on the link\n" <<
					"\tEIO\t\t5 - There was a problem retrieving or saving\n" <<
					"\t\t\tthe data\n" <<
					"\tEEXIST\t\t17 - The file already exists\n" <<
					"\tEISDIR\t\t21 - The file you are asking us to create\n" <<
					"\t\t\tis a directory\n" <<
					flush;
			cout <<
					"Keep in mind that this is, in fact, free software, developed\n" <<
					"independently and therefore comes with no warranty.\n" << flush;
			return return_code;
		}
	} else {
		return_code = EX_USAGE;
		cout << "This utility requires one of the following arguments:\n" <<
				"\t-h, --help\n" <<
				"Or one of the following sets:\n" <<
				"\t-v [filename]\n" <<
				"\t-q [filename]\n" <<
				"Alternatively -v may be --verbose or -q may be --quiet\n" << flush;
		return return_code;
	}

	// Check the filename to be sure it is good.
	struct stat st;
	bool exist = true, is_dir = true;

	is_dir = S_ISDIR(st.st_mode);
	exist = (stat(filename.c_str(), &st) == 0);
	if(exist) {
		if(verbose) cout << "File \'" << filename << "\' already exists\n";
		return_code = EEXIST;
		return return_code;
	}
	if(is_dir) {
		if(verbose) cout << "File \'" << filename << "\' is a directory\n";
		return_code = EISDIR;
		return return_code;
	}

	// We know now that filename does not already exist and it is not a directory
	// We may now continue
	if(verbose) cout << "Searching for calculator..." << flush;
	ti_usb calc;

	// Calling is_good() here verifies that there was a calculator located on
	// the USB bus and that we can in fact communicate with it.
	if(calc.is_good()) {
		if(verbose) cout << "found calculator\n";

		if(verbose) cout << "Getting screenshot\n";
		if(calc.get_screenshot(filename)) {
			if(verbose) cout << "succeeded - screenshot saved to " << filename << "\n";
			return return_code;
		}
		if(verbose) cout << "failed\n";
		return_code = EIO;
		return return_code;
	}
	if(verbose) cout << "\nNo calculator found on the USB bus\n";
	return_code = ENOTCONN;
	return return_code;
}
